/*
FullCalendar Vue Component v5.0.0-beta.1
Docs: https://fullcalendar.io/docs/vue
License: MIT
*/
import { createPlugin, Calendar } from '@fullcalendar/core';
import Vue from 'vue';

function _typeof(obj) {
  "@babel/helpers - typeof";

  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

var OPTION_IS_COMPLEX = {
  header: true,
  footer: true,
  events: true,
  eventSources: true,
  resources: true
};

/*
works with objects and arrays
*/
function shallowCopy(val) {
  if (_typeof(val) === 'object') {
    if (Array.isArray(val)) {
      val = Array.prototype.slice.call(val);
    } else if (val) {
      // non-null
      val = _objectSpread2({}, val);
    }
  }

  return val;
}
function diffProps(oldProps, newProps) {
  var updates = {};
  var removals = [];
  var anyChanges = false;

  for (var propName in newProps) {
    if (newProps[propName] !== oldProps[propName]) {
      updates[propName] = newProps[propName];
      anyChanges = true;
    }
  }

  for (var _propName in oldProps) {
    if (!(_propName in newProps)) {
      removals.push(_propName);
      anyChanges = true;
    }
  }

  return {
    updates: updates,
    removals: removals,
    anyChanges: anyChanges
  };
}
function mapHash(input, func) {
  var output = {};

  for (var key in input) {
    if (hasOwnProperty.call(input, key)) {
      output[key] = func(input[key], key);
    }
  }

  return output;
}

/*
wrap it in an object with a `vue` key, which the custom content-type handler system will look for
*/

function wrapVDomGenerator(vDomGenerator) {
  return function () {
    return {
      vue: vDomGenerator.apply(this, arguments)
    };
  };
}
var VueContentTypePlugin = createPlugin({
  contentTypeHandlers: {
    vue: buildVDomHandler // looks for the `vue` key

  }
});

function buildVDomHandler() {
  var currentEl;
  var v; // the Vue instance

  return function (el, vDomContent) {
    // the handler
    if (currentEl !== el) {
      if (currentEl && v) {
        // if changing elements, recreate the vue
        v.$destroy();
      }

      currentEl = el;
    }

    if (!v) {
      v = initVue(vDomContent); // vue's mount method *replaces* the given element. create an artificial inner el

      var innerEl = document.createElement('span');
      el.appendChild(innerEl);
      v.$mount(innerEl);
    } else {
      v.content = vDomContent;
    }
  };
}

function initVue(initialContent) {
  return new Vue({
    props: ['content'],
    propsData: {
      content: initialContent
    },
    render: function render(h) {
      var content = this.content; // the slot result can be an array, but the returned value of a vue component's
      // render method must be a single node.

      if (content.length === 1) {
        return content[0];
      } else if (content.length) {
        return h('span', {}, content);
      }
    }
  });
}

/*
IMPORTANT NOTE: `this.$options` is merely a place to store state.
The `this.options` prop holds the FullCalendar options.
*/

var FullCalendarComponent = {
  props: ['options'],
  data: function data() {
    return {
      renderId: 0
    };
  },
  render: function render(createElement) {
    return createElement('div', {
      // when renderId is changed, Vue will trigger a real-DOM async rerender, calling beforeUpdate/updated
      attrs: {
        'data-fc-render-id': this.renderId
      }
    });
  },
  mounted: function mounted() {
    var options = this.options || {};
    this.$options.calendar = new Calendar(this.$el, // the snapshot will NOT use this transformed object, so it's okay to inject new values
    buildInitialOptions(options, this.$scopedSlots));
    this.$options.calendar.render();
    this.$options.optionSnapshot = shallowCopy(options); // for diffing of non-complex options only
  },
  methods: {
    getApi: function getApi() {
      return this.$options.calendar;
    }
  },
  beforeUpdate: function beforeUpdate() {
    this.getApi().resumeRendering(); // the watcher handlers paused it
  },
  beforeDestroy: function beforeDestroy() {
    this.getApi().destroy();
  },
  watch: buildWatchers()
};

function buildWatchers() {
  var watchers = {
    // watches changes of ALL options and their nested objects,
    // but this is only a means to be notified of top-level non-complex options changes.
    options: {
      deep: true,
      handler: function handler(options) {
        var optionSnapshot = this.$options.optionSnapshot;
        var diff = diffProps(optionSnapshot, options);

        if (diff.anyChanges) {
          // update optionSnapshot for next time
          for (var optionName in diff.updates) {
            if (!OPTION_IS_COMPLEX[optionName]) {
              optionSnapshot[optionName] = options[optionName];
            }
          }

          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = diff.removals[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              var _optionName = _step.value;
              delete optionSnapshot[_optionName];
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator["return"] != null) {
                _iterator["return"]();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }

          var calendar = this.getApi();
          calendar.pauseRendering();
          calendar.mutateOptions(diff.updates, diff.removals);
          this.renderId++; // will queue a rerender
        }
      }
    }
  };

  var _loop = function _loop(complexOptionName) {
    // handlers called when nested objects change
    watchers["options.".concat(complexOptionName)] = {
      deep: true,
      handler: function handler(val) {
        // unfortunately the handler is called with undefined if new props were set, but the complex one wasn't ever set
        if (val !== undefined) {
          var calendar = this.getApi();
          calendar.pauseRendering();
          calendar.mutateOptions(_defineProperty({}, complexOptionName, shallowCopy(val)));
          this.renderId++; // will queue a rerender
        }
      }
    };
  };

  for (var complexOptionName in OPTION_IS_COMPLEX) {
    _loop(complexOptionName);
  }

  return watchers;
}

function buildInitialOptions(options, scopedSlots) {
  return _objectSpread2({}, options, {
    plugins: (options.plugins || []).concat([VueContentTypePlugin])
  }, mapHash(scopedSlots, wrapVDomGenerator));
}

/*
Registers the component globally if appropriate.
This modules exposes the component AND an install function.

Derived from:
https://vuejs.org/v2/cookbook/packaging-sfc-for-npm.html
*/

var installed = false; // declare install function executed by Vue.use()

function install(Vue) {
  if (!installed) {
    installed = true;
    Vue.component('FullCalendar', FullCalendarComponent);
  }
} // detect a globally availble version of Vue (eg. in browser via <script> tag)

var GlobalVue;

if (typeof window !== 'undefined') {
  GlobalVue = window.Vue;
} else if (typeof global !== 'undefined') {
  GlobalVue = global.Vue;
} // auto-install if possible


if (GlobalVue) {
  GlobalVue.use({
    install: install
  });
} // to allow use as module (npm/webpack/etc.) export component

export default FullCalendarComponent;
export { install };
